@@ -4,11 +4,10 @@
 # You can obtain one at http://mozilla.org/MPL/2.0/.
 
 import math
-import random
 import statistics
 import struct
 from functools import reduce
-from typing import Set
+from typing import Dict, List, Set, Tuple
 
 import numpy as np
 import xgboost
@@ -38,13 +37,11 @@ def get_commit_map():
 
 
 class TestSelectModel(Model):
-    def __init__(self, lemmatization=False, granularity="label", use_subset=False):
+    def __init__(self, lemmatization=False, granularity="label", failures_skip=None):
         Model.__init__(self, lemmatization)
 
         self.granularity = granularity
-        # This is useful for development purposes, it avoids using too much memory
-        # by using a subset of the dataset (dropping some passing runnables).
-        self.use_subset = use_subset
+        self.failures_skip = failures_skip
 
         self.training_dbs = [repository.COMMITS_DB]
         self.eval_dbs[repository.COMMITS_DB] = (
@@ -82,18 +79,6 @@ class TestSelectModel(Model):
             ]
         elif granularity == "group":
             feature_extractors += [
-                commit_features.source_code_files_modified_num(),
-                commit_features.other_files_modified_num(),
-                commit_features.test_files_modified_num(),
-                commit_features.source_code_file_size(),
-                commit_features.other_file_size(),
-                commit_features.test_file_size(),
-                commit_features.source_code_added(),
-                commit_features.other_added(),
-                commit_features.test_added(),
-                commit_features.source_code_deleted(),
-                commit_features.other_deleted(),
-                commit_features.test_deleted(),
                 test_scheduling_features.path_distance(),
                 test_scheduling_features.common_path_components(),
                 test_scheduling_features.touched_together(),
@@ -112,10 +97,9 @@ class TestSelectModel(Model):
         self.clf = xgboost.XGBClassifier(n_jobs=16)
         self.clf.set_params(predictor="cpu_predictor")
 
-    def get_pushes(self):
-        if self.use_subset:
-            random.seed(0)
-
+    def get_pushes(
+        self, apply_filters: bool = False
+    ) -> Tuple[List[Dict[str, List[str]]], float]:
         pushes = []
         for revs, test_datas in test_scheduling.get_test_scheduling_history(
             self.granularity
@@ -137,8 +121,9 @@ class TestSelectModel(Model):
                 else:
                     passes.append(name)
 
-            if self.use_subset:
-                passes = random.sample(passes, math.ceil(len(passes) / 10))
+            if apply_filters:
+                if self.failures_skip and len(failures) > self.failures_skip:
+                    continue
 
             pushes.append(
                 {"revs": revs, "failures": failures, "passes": passes,}
@@ -149,7 +134,7 @@ class TestSelectModel(Model):
     # To properly test the performance of our model, we need to split the data
     # according to time: we train on older pushes and evaluate on newer pushes.
     def train_test_split(self, X, y):
-        pushes, train_push_len = self.get_pushes()
+        pushes, train_push_len = self.get_pushes(True)
         train_len = sum(
             len(push["failures"]) + len(push["passes"])
             for push in pushes[:train_push_len]
@@ -183,7 +168,7 @@ class TestSelectModel(Model):
 
     def get_labels(self):
         classes = {}
-        pushes, _ = self.get_pushes()
+        pushes, _ = self.get_pushes(True)
 
         for push in pushes:
             for name in push["failures"]:
@@ -241,7 +226,18 @@ class TestSelectModel(Model):
     def reduce(self, tasks: Set[str], min_redundancy_confidence: float) -> Set[str]:
         failing_together = test_scheduling.get_failing_together_db()
 
-        priorities = ["linux", "win", "mac", "android"]
+        priorities1 = [
+            "tsan",
+            "android-hw",
+            "linux32",
+            "asan",
+            "mac",
+            "windows7",
+            "android-em",
+            "windows10",
+            "linux64",
+        ]
+        priorities2 = ["debug", "opt"]
 
         to_drop = set()
         to_analyze = sorted(tasks)
@@ -257,12 +253,21 @@ class TestSelectModel(Model):
                 if confidence < min_redundancy_confidence:
                     continue
 
-                for priority in priorities:
-                    if priority in task1:
-                        to_drop.add(task2)
+                for priority in priorities1:
+                    if priority in task1 and priority in task2:
+                        for priority in priorities2:
+                            if priority in task1:
+                                to_drop.add(task1)
+                                break
+                            elif priority in task2:
+                                to_drop.add(task2)
+                                break
+                        break
+                    elif priority in task1:
+                        to_drop.add(task1)
                         break
                     elif priority in task2:
-                        to_drop.add(task1)
+                        to_drop.add(task2)
                         break
 
             to_analyze = [t for t in to_analyze if t not in to_drop]
@@ -271,7 +276,7 @@ class TestSelectModel(Model):
 
     def evaluation(self):
         # Get a test set of pushes on which to test the model.
-        pushes, train_push_len = self.get_pushes()
+        pushes, train_push_len = self.get_pushes(False)
 
         test_pushes = pushes[train_push_len:]
 
@@ -280,10 +285,14 @@ class TestSelectModel(Model):
             (set(push["failures"]) | set(push["passes"]) for push in test_pushes[-28:]),
         )
 
+        test_pushes_failures = sum(
+            1 for push in test_pushes if len(push["failures"]) > 0
+        )
+
         test_pushes = {push["revs"][0]: push for push in test_pushes}
 
         print(
-            f"Testing on {len(pushes) - train_push_len} out of {len(pushes)}. {len(all_tasks)} schedulable tasks."
+            f"Testing on {len(test_pushes)} ({test_pushes_failures} with failures) out of {len(pushes)}. {len(all_tasks)} schedulable tasks."
         )
 
         commit_map = get_commit_map()
@@ -300,6 +309,7 @@ class TestSelectModel(Model):
                 if revision in commit_map
             )
             if len(commits) == 0:
+                test_pushes[rev]["all_possibly_selected"] = {}
                 continue
 
             push_num = last_push_num - (len(test_pushes) - (i + 1))
@@ -314,7 +324,7 @@ class TestSelectModel(Model):
 
         reductions = [None]
         if self.granularity == "label":
-            reductions += [0.7, 0.8, 0.9, 1.0]
+            reductions += [0.9, 1.0]
 
         def do_eval(confidence_threshold, reduction, cap):
             for rev, push in test_pushes.items():
@@ -398,9 +408,9 @@ class TestSelectModel(Model):
                 f"For confidence threshold {confidence_threshold}, with reduction {reduction_str}, and cap at {cap}: scheduled {average_scheduled} tasks on average (min {min_scheduled}, max {max_scheduled}). In {percentage_caught_one}% of pushes we caught at least one failure ({percentage_caught_one_or_some_didnt_run}% ignoring misses when some of our selected tasks didn't run). On average, we caught {average_caught_percentage}% of all seen failures."
             )
 
-        for cap in [None, 200, 300, 500]:
+        for cap in [None, 300, 500]:
             for reduction in reductions:
-                for confidence_threshold in [0.3, 0.5, 0.7, 0.8]:
+                for confidence_threshold in [0.5, 0.7, 0.8, 0.85, 0.9, 0.95]:
                     do_eval(confidence_threshold, reduction, cap)
 
     def get_feature_names(self):
@@ -409,7 +419,7 @@ class TestSelectModel(Model):
 
 class TestLabelSelectModel(TestSelectModel):
     def __init__(self, lemmatization=False):
-        TestSelectModel.__init__(self, lemmatization, "label")
+        TestSelectModel.__init__(self, lemmatization, "label", failures_skip=60)
 
 
 class TestGroupSelectModel(TestSelectModel):
