@@ -18,6 +18,7 @@ import sys
 import threading
 from datetime import datetime
 from functools import lru_cache
+from typing import Generator, Iterable, List
 
 import hglib
 import lmdb
@@ -28,6 +29,8 @@ from bugbug.utils import LMDBDict
 
 logger = logging.getLogger(__name__)
 
+code_analysis_server = None
+
 hg_servers = list()
 hg_servers_lock = threading.Lock()
 thread_local = threading.local()
@@ -37,7 +40,7 @@ COMMIT_EXPERIENCES_DB = "commit_experiences.lmdb.tar.zst"
 db.register(
     COMMITS_DB,
     "https://community-tc.services.mozilla.com/api/index/v1/task/project.relman.bugbug.data_commits.latest/artifacts/public/commits.json.zst",
-    11,
+    12,
     [COMMIT_EXPERIENCES_DB],
 )
 
@@ -90,6 +93,15 @@ TYPES_TO_EXT = {**SOURCE_CODE_TYPES_TO_EXT, **OTHER_TYPES_TO_EXT}
 EXT_TO_TYPES = {ext: typ for typ, exts in TYPES_TO_EXT.items() for ext in exts}
 
 
+def get_type(path: str) -> str:
+    file_name = os.path.basename(path)
+    if file_name in HARDCODED_TYPES:
+        return HARDCODED_TYPES[file_name]
+
+    ext = os.path.splitext(path)[1].lower()
+    return EXT_TO_TYPES.get(ext, ext)
+
+
 class Commit:
     def __init__(
         self,
@@ -99,6 +111,7 @@ class Commit:
         date,
         pushdate,
         bug_id,
+        backsout,
         backedoutby,
         author_email,
         reviewers,
@@ -110,8 +123,8 @@ class Commit:
         self.desc = desc
         self.date = date
         self.pushdate = pushdate
+        self.backsout = backsout
         self.backedoutby = backedoutby
-        self.ever_backedout = backedoutby != ""
         self.author_email = author_email
         self.reviewers = reviewers
         self.ignored = ignored
@@ -201,7 +214,7 @@ class Commit:
 
     def to_dict(self):
         d = self.__dict__
-        for f in ["backedoutby", "ignored", "file_copies"]:
+        for f in ["file_copies"]:
             del d[f]
         d["types"] = list(d["types"])
         d["pushdate"] = str(d["pushdate"])
@@ -223,8 +236,42 @@ def get_directories(files):
     return list(directories)
 
 
-def get_commits():
-    return db.read(COMMITS_DB)
+def is_wptsync(commit: dict) -> bool:
+    return commit["author_email"] == "wptsync@mozilla.com" or any(
+        s in commit["desc"] for s in ("wpt-pr:", "wpt-head:", "wpt-type:")
+    )
+
+
+def filter_commits(
+    commits: Iterable[dict],
+    include_no_bug: bool = False,
+    include_backouts: bool = False,
+    include_ignored: bool = False,
+) -> Generator[dict, None, None]:
+    for commit in commits:
+        if not include_ignored and commit["ignored"]:
+            continue
+
+        if not include_no_bug and not commit["bug_id"]:
+            continue
+
+        if not include_backouts and len(commit["backsout"]) > 0:
+            continue
+
+        yield commit
+
+
+def get_commits(
+    include_no_bug: bool = False,
+    include_backouts: bool = False,
+    include_ignored: bool = False,
+) -> Generator[dict, None, None]:
+    return filter_commits(
+        db.read(COMMITS_DB),
+        include_no_bug=include_no_bug,
+        include_backouts=include_backouts,
+        include_ignored=include_ignored,
+    )
 
 
 def _init_process(repo_dir):
@@ -430,7 +477,7 @@ def get_metrics(commit, metrics_space):
 def transform(hg, repo_dir, commit):
     hg_modified_files(hg, commit)
 
-    if commit.ignored:
+    if commit.ignored or len(commit.backsout) > 0 or commit.bug_id is None:
         return commit
 
     source_code_sizes = []
@@ -439,7 +486,12 @@ def transform(hg, repo_dir, commit):
     metrics_file_count = 0
 
     patch = hg.export(revs=[commit.node.encode("ascii")], git=True)
-    patch_data = rs_parsepatch.get_lines(patch)
+    try:
+        patch_data = rs_parsepatch.get_lines(patch)
+    except Exception:
+        logger.error(f"Exception while analyzing {commit.node}")
+        raise
+
     for stats in patch_data:
         path = stats["filename"]
 
@@ -461,12 +513,8 @@ def transform(hg, repo_dir, commit):
                 if b"no such file in rev" not in e.err:
                     raise
 
-        file_name = os.path.basename(path)
-        if file_name in HARDCODED_TYPES:
-            type_ = HARDCODED_TYPES[file_name]
-        else:
-            ext = os.path.splitext(path)[1].lower()
-            type_ = EXT_TO_TYPES.get(ext, ext)
+        type_ = get_type(path)
+        ext = os.path.splitext(path)[1].lower()
 
         if is_test(path):
             commit.test_files_modified_num += 1
@@ -578,7 +626,7 @@ def _transform(commit):
 
 
 def hg_log(hg, revs):
-    template = "{node}\\0{author}\\0{desc}\\0{date|hgdate}\\0{bug}\\0{backedoutby}\\0{author|email}\\0{pushdate|hgdate}\\0{reviewers}\\0"
+    template = "{node}\\0{author}\\0{desc}\\0{date|hgdate}\\0{bug}\\0{backedoutby}\\0{author|email}\\0{pushdate|hgdate}\\0{reviewers}\\0{backsoutnodes}\\0"
 
     args = hglib.util.cmdbuilder(
         b"log",
@@ -610,6 +658,12 @@ def hg_log(hg, revs):
             else []
         )
 
+        backsout = (
+            list(set(sys.intern(r) for r in rev[9].decode("utf-8").split(" ")))
+            if rev[9] != b""
+            else []
+        )
+
         revs.append(
             Commit(
                 node=sys.intern(rev[0].decode("ascii")),
@@ -618,6 +672,7 @@ def hg_log(hg, revs):
                 date=date,
                 pushdate=pushdate,
                 bug_id=bug_id,
+                backsout=backsout,
                 backedoutby=rev[5].decode("ascii"),
                 author_email=rev[6].decode("utf-8"),
                 reviewers=reviewers,
@@ -752,9 +807,9 @@ def calculate_experiences(commits, first_pushdate, save=True):
             # We don't want to consider backed out commits when calculating normal experiences.
             if (
                 commit_type == ""
-                and not commit.ever_backedout
+                and not commit.backedoutby
                 or commit_type == "backout"
-                and commit.ever_backedout
+                and commit.backedoutby
             ):
                 for i in range(len(items)):
                     exp_queues[i][day] = total_exps[i] + 1
@@ -817,9 +872,9 @@ def calculate_experiences(commits, first_pushdate, save=True):
             # We don't want to consider backed out commits when calculating normal experiences.
             if (
                 commit_type == ""
-                and not commit.ever_backedout
+                and not commit.backedoutby
                 or commit_type == "backout"
-                and commit.ever_backedout
+                and commit.backedoutby
             ):
                 for i in range(len(items)):
                     exp_queues[i][day] = all_commit_lists[i] + (commit.node,)
@@ -850,7 +905,11 @@ def calculate_experiences(commits, first_pushdate, save=True):
                         f"Experience missing for file {orig}, type '{commit_type}', on commit {commit.node}"
                     )
 
-        if not commit.ignored:
+        if (
+            not commit.ignored
+            and len(commit.backsout) == 0
+            and commit.bug_id is not None
+        ):
             update_experiences("author", day, [commit.author])
             update_experiences("reviewer", day, commit.reviewers)
 
@@ -859,7 +918,7 @@ def calculate_experiences(commits, first_pushdate, save=True):
             update_complex_experiences("component", day, commit.components)
 
 
-def set_commits_to_ignore(hg, repo_dir, commits, ignore_no_bug=True):
+def set_commits_to_ignore(hg: hglib.client, repo_dir: str, commits: List[Commit]):
     # Skip commits which are in .hg-annotate-ignore-revs or which have
     # 'ignore-this-changeset' in their description (mostly consisting of very
     # large and not meaningful formatting changes).
@@ -868,24 +927,10 @@ def set_commits_to_ignore(hg, repo_dir, commits, ignore_no_bug=True):
     ).decode("utf-8")
     ignore_revs = set(l[:40] for l in ignore_revs_content.splitlines())
 
-    backouts = set(commit.backedoutby for commit in commits if commit.ever_backedout)
-
-    def should_ignore(commit):
-        if commit.node in ignore_revs or "ignore-this-changeset" in commit.desc:
-            return True
-
-        # Don't analyze backouts.
-        if commit.node in backouts:
-            return True
-
-        # Don't analyze commits that are not linked to a bug.
-        if ignore_no_bug and commit.bug_id is None:
-            return True
-
-        return False
-
     for commit in commits:
-        commit.ignored = should_ignore(commit)
+        commit.ignored = (
+            commit.node in ignore_revs or "ignore-this-changeset" in commit.desc
+        )
 
 
 def download_component_mapping():
@@ -950,13 +995,15 @@ def get_first_pushdate(repo_dir):
 
 
 def download_commits(
-    repo_dir,
-    rev_start=None,
-    revs=None,
-    save=True,
-    use_single_process=False,
-    ignore_no_bug=True,
-):
+    repo_dir: str,
+    rev_start: str = None,
+    revs: List[str] = None,
+    save: bool = True,
+    use_single_process: bool = False,
+    include_no_bug: bool = False,
+    include_backouts: bool = False,
+    include_ignored: bool = False,
+) -> List[dict]:
     assert revs is not None or rev_start is not None
 
     with hglib.open(repo_dir) as hg:
@@ -980,7 +1027,7 @@ def download_commits(
             logger.info("Downloading file->component mapping...")
             download_component_mapping()
 
-        set_commits_to_ignore(hg, repo_dir, commits, ignore_no_bug)
+        set_commits_to_ignore(hg, repo_dir, commits)
 
         commits_num = len(commits)
 
@@ -1012,15 +1059,22 @@ def download_commits(
 
     logger.info(f"Applying final commits filtering...")
 
-    commits = [commit.to_dict() for commit in commits if not commit.ignored]
+    commits = [commit.to_dict() for commit in commits]
 
     if save:
         db.append(COMMITS_DB, commits)
 
-    return commits
+    return list(
+        filter_commits(
+            commits,
+            include_no_bug=include_no_bug,
+            include_backouts=include_backouts,
+            include_ignored=include_ignored,
+        )
+    )
 
 
-def clean(hg, repo_dir, pull=True):
+def clean(hg, repo_dir):
     logger.info("Restoring files to their checkout state...")
     hg.revert(repo_dir.encode("utf-8"), all=True)
 
@@ -1034,18 +1088,24 @@ def clean(hg, repo_dir, pull=True):
         if b"abort: empty revision set" not in e.err:
             raise
 
-    # Pull and update.
-    if pull:
-        logger.info(f"Pulling and updating {repo_dir}")
-        hg.pull(update=True)
-        logger.info(f"{repo_dir} pulled and updated")
-
 
 def clone(repo_dir, url="https://hg.mozilla.org/mozilla-central", update=False):
     try:
         with hglib.open(repo_dir) as hg:
             clean(hg, repo_dir)
 
+        # Remove pushlog DB to make sure it's regenerated.
+        try:
+            os.remove(os.path.join(repo_dir, ".hg", "pushlog2.db"))
+        except FileNotFoundError:
+            logger.info("pushlog database doesn't exist")
+
+        # Pull, to make sure the pushlog is generated.
+        with hglib.open(repo_dir) as hg:
+            logger.info(f"Pulling {repo_dir}")
+            hg.pull(update=update)
+            logger.info(f"{repo_dir} pulled")
+
         return
     except hglib.error.ServerError as e:
         if "abort: repository" not in str(e) and b"not found" not in str(e):
@@ -1068,16 +1128,6 @@ def clone(repo_dir, url="https://hg.mozilla.org/mozilla-central", update=False):
 
     logger.info(f"{repo_dir} cloned")
 
-    # Remove pushlog DB to make sure it's regenerated.
-    try:
-        os.remove(os.path.join(repo_dir, ".hg", "pushlog2.db"))
-    except FileNotFoundError:
-        logger.info("pushlog database doesn't exist")
-
-    # Pull and update, to make sure the pushlog is generated.
-    with hglib.open(repo_dir) as hg:
-        clean(hg, repo_dir)
-
 
 def apply_stack(repo_dir, stack, branch):
     """Apply a stack of patches on a repository"""
