@@ -12,17 +12,21 @@ import struct
 from bugbug import db, repository
 from bugbug.utils import ExpQueue, LMDBDict
 
-PUSH_DATA_URL = "https://community-tc.services.mozilla.com/api/index/v1/task/project.relman.bugbug.data_test_scheduling_history_push_data.latest/artifacts/public/push_data_{granularity}.json.zst"
-
 TEST_LABEL_SCHEDULING_DB = "data/test_label_scheduling_history.pickle"
 PAST_FAILURES_LABEL_DB = "past_failures_label.lmdb.tar.zst"
 FAILING_TOGETHER_LABEL_DB = "failing_together_label.lmdb.tar.zst"
 db.register(
     TEST_LABEL_SCHEDULING_DB,
     "https://community-tc.services.mozilla.com/api/index/v1/task/project.relman.bugbug.data_test_label_scheduling_history.latest/artifacts/public/test_label_scheduling_history.pickle.zst",
-    12,
+    13,
     [PAST_FAILURES_LABEL_DB, FAILING_TOGETHER_LABEL_DB],
 )
+PUSH_DATA_LABEL_DB = "data/push_data_label.json"
+db.register(
+    PUSH_DATA_LABEL_DB,
+    "https://community-tc.services.mozilla.com/api/index/v1/task/project.relman.bugbug.data_test_scheduling_history_push_data.latest/artifacts/public/push_data_label.json.zst",
+    1,
+)
 
 TEST_GROUP_SCHEDULING_DB = "data/test_group_scheduling_history.pickle"
 PAST_FAILURES_GROUP_DB = "past_failures_group.lmdb.tar.zst"
@@ -30,9 +34,15 @@ TOUCHED_TOGETHER_DB = "touched_together.lmdb.tar.zst"
 db.register(
     TEST_GROUP_SCHEDULING_DB,
     "https://community-tc.services.mozilla.com/api/index/v1/task/project.relman.bugbug.data_test_group_scheduling_history.latest/artifacts/public/test_group_scheduling_history.pickle.zst",
-    16,
+    18,
     [PAST_FAILURES_GROUP_DB, TOUCHED_TOGETHER_DB],
 )
+PUSH_DATA_GROUP_DB = "data/push_data_group.json"
+db.register(
+    PUSH_DATA_GROUP_DB,
+    "https://community-tc.services.mozilla.com/api/index/v1/task/project.relman.bugbug.data_test_scheduling_history_push_data.latest/artifacts/public/push_data_group.json.zst",
+    1,
+)
 
 HISTORICAL_TIMESPAN = 4500
 
@@ -156,7 +166,7 @@ def update_touched_together():
 
             # As in the test scheduling history retriever script, for now skip commits which are too large.
             # Skip backed-out commits since they are usually relanded and we don't want to count them twice.
-            if len(commit["files"]) <= 50 and not commit["ever_backedout"]:
+            if len(commit["files"]) <= 50 and not commit["backedoutby"]:
                 # Number of times a source file was touched together with a directory.
                 for f1 in commit["files"]:
                     for d2 in set(
