@@ -14,7 +14,6 @@ from datetime import datetime
 from logging import INFO, basicConfig, getLogger
 
 import dateutil.parser
-import hglib
 import tenacity
 from dateutil.relativedelta import relativedelta
 from libmozdata import vcs_map
@@ -47,32 +46,24 @@ IGNORED_COMMITS_DB = "data/ignored_commits.json"
 db.register(
     IGNORED_COMMITS_DB,
     "https://s3-us-west-2.amazonaws.com/communitytc-bugbug/data/ignored_commits.json.zst",
-    2,
+    5,
 )
 
 
 class RegressorFinder(object):
     def __init__(
         self,
-        mercurial_repo_dir=None,
         git_repo_url=None,
         git_repo_dir=None,
         tokenized_git_repo_url=None,
         tokenized_git_repo_dir=None,
     ):
-        self.mercurial_repo_dir = mercurial_repo_dir
         self.git_repo_url = git_repo_url
         self.git_repo_dir = git_repo_dir
         self.tokenized_git_repo_url = tokenized_git_repo_url
         self.tokenized_git_repo_dir = tokenized_git_repo_dir
 
         with ThreadPoolExecutorResult(max_workers=3) as executor:
-            if self.mercurial_repo_dir is not None:
-                logger.info(
-                    f"Cloning mercurial repository to {self.mercurial_repo_dir}..."
-                )
-                executor.submit(repository.clone, self.mercurial_repo_dir)
-
             if self.git_repo_url is not None:
                 logger.info(f"Cloning {self.git_repo_url} to {self.git_repo_dir}...")
                 executor.submit(
@@ -128,56 +119,48 @@ class RegressorFinder(object):
                 self.mercurial_to_tokenized_git,
             ) = microannotate_utils.get_commit_mapping(self.tokenized_git_repo_dir)
 
-    # TODO: Make repository module analyze all commits, even those to ignore, but add a field "ignore" or a function should_ignore that analyzes the commit data. This way we don't have to clone the Mercurial repository in this script.
-    def get_commits_to_ignore(self):
-        logger.info("Download previous commits to ignore...")
-        db.download(IGNORED_COMMITS_DB)
+    def get_commits_to_ignore(self) -> None:
+        assert db.download(repository.COMMITS_DB)
 
-        logger.info("Get previously classified commits...")
-        prev_commits_to_ignore = list(db.read(IGNORED_COMMITS_DB))
-        logger.info(f"Already found {len(prev_commits_to_ignore)} commits to ignore...")
-
-        # When we already have some analyzed commits, re-analyze the last 3500 to make sure
-        # we didn't miss back-outs that happened since the last analysis.
-        if len(prev_commits_to_ignore) > 0:
-            first_commit_to_reanalyze = (
-                -3500 if len(prev_commits_to_ignore) >= 3500 else 0
-            )
-            rev_start = "children({})".format(
-                prev_commits_to_ignore[first_commit_to_reanalyze]["rev"]
-            )
-        else:
-            rev_start = 0
-
-        with hglib.open(self.mercurial_repo_dir) as hg:
-            revs = repository.get_revs(hg, rev_start)
-
-        commits = repository.hg_log_multi(self.mercurial_repo_dir, revs)
-
-        with hglib.open(self.mercurial_repo_dir) as hg:
-            repository.set_commits_to_ignore(hg, self.mercurial_repo_dir, commits)
-
-        for commit in commits:
-            commit.ignored |= commit.author_email == "wptsync@mozilla.com"
-
-        chosen_commits = set()
+        ignored = set()
         commits_to_ignore = []
-        for commit in commits:
-            if commit.ignored or commit.backedoutby:
+        all_commits = set()
+
+        for commit in repository.get_commits(
+            include_no_bug=True, include_backouts=True, include_ignored=True
+        ):
+            all_commits.add(commit["node"][:12])
+
+            if (
+                commit["ignored"]
+                or commit["backedoutby"]
+                or not commit["bug_id"]
+                or len(commit["backsout"]) > 0
+                or repository.is_wptsync(commit)
+            ):
                 commits_to_ignore.append(
                     {
-                        "rev": commit.node,
-                        "type": "backedout" if commit.backedoutby else "",
+                        "rev": commit["node"],
+                        "type": "backedout" if commit["backedoutby"] else "",
                     }
                 )
-                chosen_commits.add(commit.node)
+                ignored.add(commit["node"][:12])
 
-        logger.info(f"{len(commits_to_ignore)} new commits to ignore...")
+            if len(commit["backsout"]) > 0:
+                for backedout in commit["backsout"]:
+                    if backedout[:12] in ignored:
+                        continue
+                    ignored.add(backedout[:12])
 
-        for prev_commit in prev_commits_to_ignore[::-1]:
-            if prev_commit["rev"] not in chosen_commits:
-                commits_to_ignore.append(prev_commit)
-                chosen_commits.add(prev_commit["rev"])
+                    commits_to_ignore.append({"rev": backedout, "type": "backedout"})
+
+        logger.info(f"{len(commits_to_ignore)} commits to ignore...")
+
+        # Skip backed-out commits which aren't in the repository (commits which landed *before* the Mercurial history
+        # started, and backouts which mentioned a bad hash in their message).
+        commits_to_ignore = [
+            c for c in commits_to_ignore if c["rev"][:12] in all_commits
+        ]
 
         logger.info(f"{len(commits_to_ignore)} commits to ignore...")
 
@@ -306,17 +289,17 @@ class RegressorFinder(object):
         else:
             db_path = BUG_INTRODUCING_COMMITS_DB
 
-        def git_to_mercurial(rev):
+        def git_to_mercurial(revs):
             if tokenized:
-                return self.tokenized_git_to_mercurial[rev]
+                return (self.tokenized_git_to_mercurial[rev] for rev in revs)
             else:
-                return vcs_map.git_to_mercurial(repo_dir, rev)
+                yield from vcs_map.git_to_mercurial(repo_dir, revs)
 
-        def mercurial_to_git(rev):
+        def mercurial_to_git(revs):
             if tokenized:
-                return self.mercurial_to_tokenized_git[rev]
+                return (self.mercurial_to_tokenized_git[rev] for rev in revs)
             else:
-                return vcs_map.mercurial_to_git(repo_dir, rev)
+                yield from vcs_map.mercurial_to_git(repo_dir, revs)
 
         logger.info("Download previously found bug-introducing commits...")
         db.download(db_path)
@@ -334,11 +317,12 @@ class RegressorFinder(object):
         hashes_to_ignore = set(commit["rev"] for commit in commits_to_ignore)
 
         with open("git_hashes_to_ignore", "w") as f:
-            f.writelines(
-                "{}\n".format(mercurial_to_git(commit["rev"]))
-                for commit in commits_to_ignore
+            git_hashes = mercurial_to_git(
+                commit["rev"]
+                for commit in tqdm(commits_to_ignore)
                 if not tokenized or commit["rev"] in self.mercurial_to_tokenized_git
             )
+            f.writelines("{}\n".format(git_hash) for git_hash in git_hashes)
 
         logger.info(f"{len(bug_fixing_commits)} commits to analyze")
 
@@ -383,7 +367,7 @@ class RegressorFinder(object):
         def find_bic(bug_fixing_commit):
             logger.info("Analyzing {}...".format(bug_fixing_commit["rev"]))
 
-            git_fix_revision = mercurial_to_git(bug_fixing_commit["rev"])
+            git_fix_revision = next(mercurial_to_git([bug_fixing_commit["rev"]]))
 
             commit = thread_local.git.get_commit(git_fix_revision)
 
@@ -405,10 +389,14 @@ class RegressorFinder(object):
 
             bug_introducing_modifications = {}
             for modification in commit.modifications:
-                if (
-                    get_modification_path(modification)
-                    == "testing/web-platform/meta/MANIFEST.json"
-                ):
+                path = get_modification_path(modification)
+
+                if path == "testing/web-platform/meta/MANIFEST.json":
+                    continue
+
+                # Don't try to find the bug-introducing commit for modifications
+                # in the bug-fixing commit to non-source code files.
+                if repository.get_type(path) not in repository.SOURCE_CODE_TYPES_TO_EXT:
                     continue
 
                 bug_introducing_modifications.update(
@@ -432,8 +420,8 @@ class RegressorFinder(object):
                         bug_introducing_commits.append(
                             {
                                 "bug_fixing_rev": bug_fixing_commit["rev"],
-                                "bug_introducing_rev": git_to_mercurial(
-                                    bug_introducing_hash
+                                "bug_introducing_rev": next(
+                                    git_to_mercurial([bug_introducing_hash])
                                 ),
                             }
                         )
@@ -597,15 +585,11 @@ def evaluate(bug_introducing_commits):
     )
 
 
-def main():
+def main() -> None:
     description = "Find bug-introducing commits from bug-fixing commits"
     parser = argparse.ArgumentParser(description=description)
 
     parser.add_argument("what", choices=["to_ignore", "bug_fixing", "bug_introducing"])
-    parser.add_argument(
-        "--repo_dir",
-        help="Path to a Gecko repository. If no repository exists, it will be cloned to this location.",
-    )
     parser.add_argument(
         "--git_repo_url", help="URL to the git repository on which to run SZZ."
     )
@@ -624,7 +608,6 @@ def main():
     args = parser.parse_args()
 
     regressor_finder = RegressorFinder(
-        args.repo_dir,
         args.git_repo_url,
         args.git_repo_dir,
         args.tokenized_git_repo_url,
