@@ -5,34 +5,28 @@
 
 import argparse
 import collections
+import concurrent.futures
 import itertools
-import json
 import math
 import os
-import pickle
 import struct
-import threading
-import time
 import traceback
 from datetime import datetime
 from logging import INFO, basicConfig, getLogger
+from typing import Dict, Generator, List, NewType, Tuple
 
 import adr
 import dateutil.parser
 import mozci.push
-import zstandard
-from cachy.serializers import Serializer
 from dateutil.relativedelta import relativedelta
 from tqdm import tqdm
 
 from bugbug import commit_features, db, repository, test_scheduling
-from bugbug.utils import (
-    create_tar_zst,
-    download_check_etag,
-    open_tar_zst,
-    zstd_compress,
-    zstd_decompress,
-)
+from bugbug.utils import create_tar_zst, zstd_compress
+
+Revision = NewType("Revision", str)
+TaskName = NewType("TaskName", str)
+PushResult = Tuple[List[Revision], List[TaskName], List[TaskName], List[TaskName]]
 
 basicConfig(level=INFO)
 logger = getLogger(__name__)
@@ -40,21 +34,19 @@ logger = getLogger(__name__)
 JOBS_TO_CONSIDER = ("test-", "build-")
 JOBS_TO_IGNORE = (
     "build-docker-image-",
-    "-test-verify-",
-    "-test-coverage-",
     "-android-hw-",
     "-awsy-",
     "-raptor-",
     "-talos-",
     "backlog",
+    # inclusive test suites -- these *only* run when certain files have changed
+    "-test-verify-",
+    "-test-coverage-",
+    "jittest",
+    "jsreftest",
+    "android-hw-gfx",
 )
 
-ADR_CACHE_DB = "data/adr_cache.tar"
-db.register(
-    ADR_CACHE_DB,
-    "https://s3-us-west-2.amazonaws.com/communitytc-bugbug/data/adr_cache.tar.zst",
-    3,
-)
 # The mozci version (to bump whenever we change the mozci regression algorithm),
 # so we can keep track of which version of mozci was used to analyze a given push
 # and we can decide when we want to regenerate parts of the dataset.
@@ -86,35 +78,8 @@ def rename_tasks(tasks):
     return [task.replace("test-linux64-", "test-linux1804-64-") for task in tasks]
 
 
-class CompressedPickleSerializer(Serializer):
-    def __init__(self):
-        self.compressor = zstandard.ZstdCompressor(
-            level=zstandard.MAX_COMPRESSION_LEVEL
-        )
-        self.decompressor = zstandard.ZstdDecompressor()
-
-    def serialize(self, data):
-        return self.compressor.compress(pickle.dumps(data))
-
-    def unserialize(self, data):
-        return pickle.loads(self.decompressor.decompress(data))
-
-
 class Retriever(object):
-    def __init__(self):
-        os.makedirs("data", exist_ok=True)
-
-    def upload_adr_cache(self):
-        cache_path = os.path.splitext(ADR_CACHE_DB)[0]
-        assert os.path.abspath(
-            adr.config["cache"]["stores"]["file"]["path"]
-        ) == os.path.abspath(cache_path)
-
-        create_tar_zst(f"{ADR_CACHE_DB}.zst")
-
-        db.upload(ADR_CACHE_DB)
-
-    def generate_push_data(self, runnable):
+    def generate_push_data(self, granularity: str) -> None:
         # We keep in the cache the fact that we failed to analyze a push for 10
         # days, so if we re-run often we don't retry the same pushes many times.
         MISSING_CACHE_RETENTION = 10 * 24 * 60
@@ -122,8 +87,8 @@ class Retriever(object):
         # We'll use the past TRAINING_MONTHS months only for training the model,
         # but we use half TRAINING_MONTHS months more than that to calculate the
         # failure statistics.
-        from_months = TRAINING_MONTHS[runnable] + math.floor(
-            TRAINING_MONTHS[runnable] / 2
+        from_months = TRAINING_MONTHS[granularity] + math.floor(
+            TRAINING_MONTHS[granularity] / 2
         )
 
         # We use the actual date instead of 'today-X' aliases to avoid adr caching
@@ -137,138 +102,115 @@ class Retriever(object):
             branch="autoland",
         )
 
-        num_cached = 0
+        if granularity == "label":
+            push_data_db = test_scheduling.PUSH_DATA_LABEL_DB
+        elif granularity == "group":
+            push_data_db = test_scheduling.PUSH_DATA_GROUP_DB
 
-        push_data = []
+        cache: Dict[mozci.push.Push, Tuple[PushResult, int]] = {}
 
-        def cache_key(push):
-            return f"push_data.{runnable}.{push.rev}"
+        def cache_key(push: mozci.push.Push) -> str:
+            return f"push_data.{granularity}.{push.rev}"
+
+        with concurrent.futures.ThreadPoolExecutor() as executor:
+            future_to_push = {
+                executor.submit(
+                    lambda push: adr.config.cache.get(cache_key(push)), push
+                ): push
+                for push in pushes
+            }
+
+            for future in tqdm(
+                concurrent.futures.as_completed(future_to_push),
+                total=len(future_to_push),
+            ):
+                push = future_to_push[future]
+
+                exc = future.exception()
+                if exc is not None:
+                    logger.info(f"Exception {exc} while getting {push.rev}")
+                    for f in future_to_push.keys():
+                        f.cancel()
+
+                cache[push] = future.result()
 
         # Regenerating a large amount of data when we update the mozci regression detection
         # algorithm is currently pretty slow, so we only regenerate 1000 pushes whenever we
         # run.
-        to_regenerate = set()
-        """for push in pushes[::-1]:
-            cached = adr.config.cache.get(cache_key(push))
+        """to_regenerate = 0
+        for push in pushes[::-1]:
+            cached = cache[push]
             if not cached:
                 continue
 
             value, mozci_version = cached
-            if mozci_version != MOZCI_VERSION and len(to_regenerate) < 1000:
-                to_regenerate.add(value[0][0])"""
+            if mozci_version != MOZCI_VERSION and to_regenerate < 1000:
+                cache[push] = None
+                to_regenerate += 1"""
 
-        def periodically_upload_adr_cache():
-            start_time = time.monotonic()
-            while not upload_thread_stop.isSet():
-                if time.monotonic() - start_time >= 10800:
-                    self.upload_adr_cache()
-                    start_time = time.monotonic()
+        to_regenerate = 0
+        for push in pushes[::-1]:
+            cached = cache[push]
+            if not cached:
+                continue
 
-                upload_thread_stop.wait(timeout=7)
+            if to_regenerate < 1000:
+                del cache[push]
+                adr.config.cache.put(push.push_uuid, {}, 0)
+                to_regenerate += 1
 
-        upload_thread = threading.Thread(target=periodically_upload_adr_cache)
-        upload_thread_stop = threading.Event()
-        upload_thread.start()
+        def generate() -> Generator[PushResult, None, None]:
+            num_cached = 0
 
-        s3_store = adr.util.cache_stores.S3Store(
-            {"bucket": "communitytc-bugbug", "prefix": "data/adr_cache/",}
-        )
+            for push in tqdm(pushes):
+                key = cache_key(push)
 
-        s3_store.set_serializer(CompressedPickleSerializer())
+                if push in cache and cache[push] is not None:
+                    num_cached += 1
+                    cached = cache[push]
+                    if cached:
+                        value, mozci_version = cached
+                        yield value
+                else:
+                    logger.info(f"Analyzing {push.rev} at the {granularity} level...")
 
-        for push in tqdm(pushes):
-            key = cache_key(push)
+                    try:
+                        if granularity == "label":
+                            runnables = push.task_labels
+                        elif granularity == "group":
+                            runnables = push.group_summaries.keys()
 
-            if adr.config.cache.has(key) and push.revs[0] not in to_regenerate:
-                num_cached += 1
-                cached = adr.config.cache.get(key)
-                if cached:
-                    s3_store.put(key, cached, adr.config["cache"]["retention"])
-                    value, mozci_version = cached
-                    push_data.append(value)
-            else:
-                logger.info(f"Analyzing {push.rev} at the {runnable} level...")
+                        value = (
+                            push.revs,
+                            list(runnables),
+                            list(push.get_possible_regressions(granularity)),
+                            list(push.get_likely_regressions(granularity)),
+                        )
+                        adr.config.cache.put(
+                            key,
+                            (value, MOZCI_VERSION),
+                            adr.config["cache"]["retention"],
+                        )
+                        yield value
+                    except adr.errors.MissingDataError:
+                        logger.warning(
+                            f"Tasks for push {push.rev} can't be found on ActiveData"
+                        )
+                        adr.config.cache.put(key, (), MISSING_CACHE_RETENTION)
+                    except Exception:
+                        traceback.print_exc()
+                        adr.config.cache.put(key, (), MISSING_CACHE_RETENTION)
 
-                try:
-                    if runnable == "label":
-                        runnables = push.task_labels
-                    elif runnable == "group":
-                        runnables = push.group_summaries.keys()
+            logger.info(f"{num_cached} pushes were already cached out of {len(pushes)}")
 
-                    value = [
-                        push.revs,
-                        list(runnables),
-                        list(push.get_possible_regressions(runnable)),
-                        list(push.get_likely_regressions(runnable)),
-                    ]
-                    push_data.append(value)
-                    adr.config.cache.put(
-                        key, (value, MOZCI_VERSION), adr.config["cache"]["retention"]
-                    )
-                    s3_store.put(
-                        key, (value, MOZCI_VERSION), adr.config["cache"]["retention"]
-                    )
-                except adr.errors.MissingDataError:
-                    logger.warning(
-                        f"Tasks for push {push.rev} can't be found on ActiveData"
-                    )
-                    adr.config.cache.put(key, (), MISSING_CACHE_RETENTION)
-                except Exception:
-                    traceback.print_exc()
-                    adr.config.cache.put(key, (), MISSING_CACHE_RETENTION)
-
-        upload_thread_stop.set()
-        upload_thread.join()
-
-        logger.info(f"{num_cached} pushes were already cached out of {len(pushes)}")
-
-        with open(f"push_data_{runnable}.json", "w") as f:
-            json.dump(push_data, f)
-
-        zstd_compress(f"push_data_{runnable}.json")
-
-    def retrieve_push_data(self):
-        # Download previous cache.
-        db.download(ADR_CACHE_DB, extract=False)
-
-        # Extract files from the cache.
-        with open_tar_zst(f"{ADR_CACHE_DB}.zst", "r") as tar:
-            now = round(time.time())
-
-            for member in tar:
-                if member.isdir():
-                    os.mkdir(member.name)
-                    continue
-
-                fin = tar.extractfile(member)
-
-                # If the element expired (the format of the file can be inferred
-                # from cachy's FileStore source code), no need to extract it.
-                content = fin.read(10)
-                expire = int(content)
-                if now >= expire:
-                    continue
-
-                content += fin.read()
-
-                with open(member.name, "wb") as fout:
-                    fout.write(content)
+        db.write(push_data_db, generate())
+        zstd_compress(push_data_db)
 
+    def retrieve_push_data(self) -> None:
         self.generate_push_data("label")
         self.generate_push_data("group")
 
-        self.upload_adr_cache()
-
     def generate_test_scheduling_history(self, granularity):
-        push_data_path = f"push_data_{granularity}.json"
-        updated = download_check_etag(
-            test_scheduling.PUSH_DATA_URL.format(granularity=granularity)
-        )
-        if updated:
-            zstd_decompress(push_data_path)
-            os.remove(f"{push_data_path}.zst")
-        assert os.path.exists(push_data_path), "Decompressed push data file exists"
-
         # Get the commits DB.
         assert db.download(repository.COMMITS_DB)
 
@@ -277,6 +219,7 @@ class Retriever(object):
         )
 
         if granularity == "label":
+            push_data_db = test_scheduling.PUSH_DATA_LABEL_DB
             test_scheduling_db = test_scheduling.TEST_LABEL_SCHEDULING_DB
             past_failures_db = os.path.join(
                 "data", test_scheduling.PAST_FAILURES_LABEL_DB
@@ -285,6 +228,7 @@ class Retriever(object):
                 "data", test_scheduling.FAILING_TOGETHER_LABEL_DB
             )
         elif granularity == "group":
+            push_data_db = test_scheduling.PUSH_DATA_GROUP_DB
             test_scheduling_db = test_scheduling.TEST_GROUP_SCHEDULING_DB
             past_failures_db = os.path.join(
                 "data", test_scheduling.PAST_FAILURES_GROUP_DB
@@ -293,6 +237,8 @@ class Retriever(object):
                 "data", test_scheduling.TOUCHED_TOGETHER_DB
             )
 
+        assert db.download(push_data_db)
+
         db.download(test_scheduling_db, support_files_too=True)
 
         last_node = None
@@ -409,8 +355,7 @@ class Retriever(object):
 
                 commit_map[commit_data["node"]] = commit_data
 
-            with open(push_data_path, "r") as f:
-                push_data = json.load(f)
+            push_data = list(db.read(push_data_db))
 
             logger.info(f"push data nodes: {len(push_data)}")
 
